from advertising import AbstractRequest, NoArgsRequest


class GetAdGroupRequest(NoArgsRequest):
    def __init__(self, ad_group_id: str):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.api = "/v2/sp/adGroups/%s" % self.ad_group_id
        self.method = "GET"


class GetAdGroupExtendedRequest(NoArgsRequest):
    def __init__(self, ad_group_id: str):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.api = "/v2/sp/adGroups/extended/%s" % self.ad_group_id
        self.method = "GET"


class CreateAdGroupsRequest(AbstractRequest):
    def __init__(self, ad_groups: list = None):
        super.__init__()
        self.ad_groups = ad_groups and ad_groups or []
        self.api = "/v2/sp/adGroups"
        self.method = "POST"

    def add_ad_group(self, ad_group: dict):
        self.ad_groups.append(ad_group)

    def build_args(self):
        return self.ad_groups


class UpdateAdGroupsRequest(CreateAdGroupsRequest):
    def __init__(self, ad_groups: list = None):
        super.__init__(ad_groups=ad_groups)
        self.method = "PUT"


class ArchiveAdGroupRequest(AbstractRequest):
    def __init__(self, ad_group_id: str):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.api = "/v2/sp/adGroups/%s" % self.ad_group_id
        self.method = "DELETE"


class ListAdGroupsRequest(AbstractRequest):
    def __init__(self,
                 name: str = None,
                 campaign_type: str = None,
                 campaign_ids: list = None,
                 ad_group_ids: list = None,
                 states: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__()
        self.name = name
        self.campaign_type = campaign_type
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.ad_group_ids = ad_group_ids and ad_group_ids or []
        self.states = states and states or []
        self.start_index = start_index
        self.count = count
        self.api = "/v2/sp/adGroups"
        self.method = "GET"

    def build_args(self):

        return {
            "startIndex": self.start_index,
            "count": self.count,
            "name": self.name,
            "campaignType": self.campaign_type,
            "stateFilter": ",".join(self.states),
            "campaignIdFilter":  ",".join(self.campaign_ids),
            "adGroupIdFilter":  ",".join(self.ad_group_ids),
        }


class ListAdGroupsExtendedRequest(ListAdGroupsRequest):
    def __init__(self,
                 name: str = None,
                 campaign_type: str = None,
                 campaign_ids: list = None,
                 ad_group_ids: list = None,
                 states: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__(name=name, campaign_type=campaign_type, campaign_ids=campaign_ids,
                         ad_group_ids=ad_group_ids, states=states, start_index=start_index, count=count)
        self.api = "/v2/sp/adGroups/extended"



from advertising import AbstractRequest, NoArgsRequest

__all__ = ["GetAdGroupSuggestedKeywords",
           "GetAdGroupSuggestedKeywordsExtended", "BulkAsinSuggestedKeywords"]


class GetAdGroupSuggestedKeywords(NoArgsRequest):

    def __init__(self, ad_group_id: str):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.api = "/v2/sp/adGroups/%s/suggested/keywords" % self.ad_group_id
        self.method = "GET"


class GetAdGroupSuggestedKeywordsExtended(NoArgsRequest):

    def __init__(self, ad_group_id: str):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.api = "/v2/sp/adGroups/%s/suggested/keywords/extended" % self.ad_group_id
        self.method = "GET"


class BulkAsinSuggestedKeywords(AbstractRequest):

    def __init__(self, asins: list, max_num_suggestion: int = 100):
        super().__init__()
        self.asins = asins
        self.max_num_suggestion = max_num_suggestion
        self.api = "/v2/sp/asins/suggested/keywords"
        self.method = "POST"

    def build_args(self):
        return {
            "asins": self.asins,
            "maxNumSuggestions": self.max_num_suggestion
        }



from abc import ABCMeta, abstractmethod
import os
import requests
import shutil
import gzip
import io
import tempfile


class AbstractRequest(metaclass=ABCMeta):
    api = None
    method = None
    stream = False
    files = []

    @abstractmethod
    def build_args(self):
        return None


class NoArgsRequest(AbstractRequest):
    def build_args(self):
        return None


class DownloadRequest(AbstractRequest):

    dest_file = None
    unzip = False

    def download(self, resp: requests.Response):
        temp_file_name = self.dest_file
        if self.unzip == True:
            with tempfile.NamedTemporaryFile() as tmpfile:
                temp_file_name = tmpfile.name

        with open(temp_file_name, 'wb') as f:
            for chunk in resp.iter_content(chunk_size=8192):
                f.write(chunk)

        if self.unzip == True:
            with gzip.open(temp_file_name, 'rb') as f_in:
                with open(self.dest_file, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)

        return self.dest_file

    def build_response(self, resp: requests.Response):
        if resp.status_code == 200 and self.dest_file is not None:
            self.download(resp)
        return resp



from advertising import AbstractRequest, DownloadRequest, NoArgsRequest

__all__ = ["GetReport", "RequestReport",
           "RequestAsinsReport", "DownloadReport", "ALLOW_REPORT_TYPES"]

ALLOW_REPORT_TYPES = [
    "campaigns",
    "adGroups",
    "keywords",
    "productAds",
    "targets"
]


DEFAULT_REPORT_METRICS = {
    "campaigns":  [
        "bidPlus",
        "campaignName",
        "campaignId",
        "campaignStatus",
        "campaignBudget",
        "impressions",
        "clicks",
        "cost",
        "attributedConversions1d",
        "attributedConversions7d",
        "attributedConversions14d",
        "attributedConversions30d",
        "attributedConversions1dSameSKU",
        "attributedConversions7dSameSKU",
        "attributedConversions14dSameSKU",
        "attributedConversions30dSameSKU",
        "attributedUnitsOrdered1d",
        "attributedUnitsOrdered7d",
        "attributedUnitsOrdered14d",
        "attributedUnitsOrdered30d",
        "attributedSales1d",
        "attributedSales7d",
        "attributedSales14d",
        "attributedSales30d",
        "attributedSales1dSameSKU",
        "attributedSales7dSameSKU",
        "attributedSales14dSameSKU",
        "attributedSales30dSameSKU",
        "attributedUnitsOrdered1dSameSKU",
        "attributedUnitsOrdered7dSameSKU",
        "attributedUnitsOrdered14dSameSKU",
        "attributedUnitsOrdered30dSameSKU",
    ],
    "productAds": [
        # "bidPlus",
        "campaignName",
        "campaignId",
        "adGroupName",
        "adGroupId",
        "impressions",
        "clicks",
        "cost",
        "currency",
        "asin",
        "sku",
        "attributedConversions1d",
        "attributedConversions7d",
        "attributedConversions14d",
        "attributedConversions30d",
        "attributedConversions1dSameSKU",
        "attributedConversions7dSameSKU",
        "attributedConversions14dSameSKU",
        "attributedConversions30dSameSKU",
        "attributedUnitsOrdered1d",
        "attributedUnitsOrdered7d",
        "attributedUnitsOrdered14d",
        "attributedUnitsOrdered30d",
        "attributedSales1d",
        "attributedSales7d",
        "attributedSales14d",
        "attributedSales30d",
        "attributedSales1dSameSKU",
        "attributedSales7dSameSKU",
        "attributedSales14dSameSKU",
        "attributedSales30dSameSKU",
        "attributedUnitsOrdered1dSameSKU",
        "attributedUnitsOrdered7dSameSKU",
        "attributedUnitsOrdered14dSameSKU",
        "attributedUnitsOrdered30dSameSKU",
    ],
    "adGroups": [
        "campaignName",
        "campaignId",
        "adGroupName",
        "adGroupId",
        "impressions",
        "clicks",
        "cost",
        "attributedConversions1d",
        "attributedConversions7d",
        "attributedConversions14d",
        "attributedConversions30d",
        "attributedConversions1dSameSKU",
        "attributedConversions7dSameSKU",
        "attributedConversions14dSameSKU",
        "attributedConversions30dSameSKU",
        "attributedUnitsOrdered1d",
        "attributedUnitsOrdered7d",
        "attributedUnitsOrdered14d",
        "attributedUnitsOrdered30d",
        "attributedSales1d",
        "attributedSales7d",
        "attributedSales14d",
        "attributedSales30d",
        "attributedSales1dSameSKU",
        "attributedSales7dSameSKU",
        "attributedSales14dSameSKU",
        "attributedSales30dSameSKU",
        "attributedUnitsOrdered1dSameSKU",
        "attributedUnitsOrdered7dSameSKU",
        "attributedUnitsOrdered14dSameSKU",
        "attributedUnitsOrdered30dSameSKU",
    ],
    "keywords": [
        "campaignName",
        "campaignId",
        "adGroupName",
        "adGroupId",
        "keywordId",
        "keywordText",
        "matchType",
        "impressions",
        "clicks",
        "cost",
        "attributedConversions1d",
        "attributedConversions7d",
        "attributedConversions14d",
        "attributedConversions30d",
        "attributedConversions1dSameSKU",
        "attributedConversions7dSameSKU",
        "attributedConversions14dSameSKU",
        "attributedConversions30dSameSKU",
        "attributedUnitsOrdered1d",
        "attributedUnitsOrdered7d",
        "attributedUnitsOrdered14d",
        "attributedUnitsOrdered30d",
        "attributedSales1d",
        "attributedSales7d",
        "attributedSales14d",
        "attributedSales30d",
        "attributedSales1dSameSKU",
        "attributedSales7dSameSKU",
        "attributedSales14dSameSKU",
        "attributedSales30dSameSKU",
        "attributedUnitsOrdered1dSameSKU",
        "attributedUnitsOrdered7dSameSKU",
        "attributedUnitsOrdered14dSameSKU",
        "attributedUnitsOrdered30dSameSKU",
    ],
    "targets": [
        "campaignName",
        "campaignId",
        "adGroupName",
        "adGroupId",
        "targetId",
        "targetingExpression",
        "targetingText",
        "targetingType",
        "impressions",
        "clicks",
        "cost",
        "attributedConversions1d",
        "attributedConversions7d",
        "attributedConversions14d",
        "attributedConversions30d",
        "attributedConversions1dSameSKU",
        "attributedConversions7dSameSKU",
        "attributedConversions14dSameSKU",
        "attributedConversions30dSameSKU",
        "attributedUnitsOrdered1d",
        "attributedUnitsOrdered7d",
        "attributedUnitsOrdered14d",
        "attributedUnitsOrdered30d",
        "attributedSales1d",
        "attributedSales7d",
        "attributedSales14d",
        "attributedSales30d",
        "attributedSales1dSameSKU",
        "attributedSales7dSameSKU",
        "attributedSales14dSameSKU",
        "attributedSales30dSameSKU",
        "attributedUnitsOrdered1dSameSKU",
        "attributedUnitsOrdered7dSameSKU",
        "attributedUnitsOrdered14dSameSKU",
        "attributedUnitsOrdered30dSameSKU",
    ],
    "asins": [
        "campaignName",
        "campaignId",
        "adGroupName",
        "adGroupId",
        "keywordId",
        "keywordText",
        "asin",
        "otherAsin",
        "sku",
        "currency",
        "matchType",
        "attributedUnitsOrdered1d",
        "attributedUnitsOrdered7d",
        "attributedUnitsOrdered14d",
        "attributedUnitsOrdered30d",
        "attributedUnitsOrdered1dOtherSKU",
        "attributedUnitsOrdered7dOtherSKU",
        "attributedUnitsOrdered14dOtherSKU",
        "attributedUnitsOrdered30dOtherSKU",
        "attributedSales1dOtherSKU",
        "attributedSales7dOtherSKU",
        "attributedSales14dOtherSKU",
        "attributedSales30dOtherSKU",
    ]
}


DEFAULT_REPORT_DIMENSIONAL = {
    "keywords": "query",
    "campaigns": "placement"
}


class GetReport(NoArgsRequest):

    def __init__(self, report_id: str):
        super().__init__()
        self.report_id = report_id
        self.api = "/v2/reports/%s" % self.report_id
        self.method = "GET"


class RequestReport(AbstractRequest):

    def __init__(self, record_type: str, report_date: str, metrics: list = None,  segment: str = None, account_type: str = "seller"):
        super().__init__()
        self.record_type = record_type
        self.account_type = account_type
        self.report_date = report_date
        self.metrics = metrics and metrics or DEFAULT_REPORT_METRICS.get(
            self.record_type)

        if account_type == "vendor" and self.record_type in ["productAds", "asins"]:
            self.metrics.remove("sku")

        self.segment = segment
        self.api = "/v2/sp/%s/report" % self.record_type
        self.method = "POST"

    def build_args(self):
        return {
            "reportDate": self.report_date,
            "metrics": ",".join(self.metrics),
            "segment": self.segment
        }


class RequestAsinsReport(RequestReport):

    def __init__(self,  report_date: str, campaign_type: str = "sponsoredProducts", metrics: list = None, account_type: str = "seller"):
        super().__init__(report_date=report_date, record_type="asins",
                         metrics=metrics, account_type=account_type)
        self.campaign_type = campaign_type
        self.api = "/v2/asins/report"

    def build_args(self):
        return {
            "reportDate": self.report_date,
            "metrics": ",".join(self.metrics),
            "campaignType": self.campaign_type
        }


class DownloadReport(DownloadRequest, NoArgsRequest):
    def __init__(self, report_id: str, dest_file: str = None, unzip=True):

        super().__init__()
        self.report_id = report_id
        self.dest_file = dest_file
        self.api = "/v2/reports/%s/download" % self.report_id
        self.method = "GET"
        self.unzip = unzip



import requests
from .config import get_endpoint


class Auth(object):

    def __init__(self, client_id: str,  client_secret: str, redirect_uri: str, marketplace="US", scope: str = "cpc_advertising:campaign_management"):
        """
        scope:
            The OAuth 2.0 permission scope used to limit the application's access to an advertiser's account.
            For the Sponsored Brands, Sponsored Display, and Sponsored Products APIs, set scope to cpc_advertising:campaign_management.
            For the Data Provider API, set scope to advertising::audiences.
            For the DSP API, set scope to advertising::campaign_management.
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.scope = scope
        # get api endpoint
        self.endpoints = get_endpoint(marketplace)
        self.marketplace = marketplace

    @property
    def authorization_url(self):
        return self.endpoints.get("AUTHORIZATION")

    def get_token(self, grant_type: str, code: str):
        """
        grant_type
            refresh_token
            authorization_code

        """

        token_url = self.endpoints.get("TOKEN")

        data = {
            "grant_type": grant_type,
            "client_id": self.client_id,
            "client_secret": self.client_secret
        }

        if grant_type == "refresh_token":
            data["refresh_token"] = code
        else:
            data["code"] = code

        r = requests.post(token_url, data=data)

        if r.status_code == 200:
            return r.json()



from advertising import AbstractRequest, DownloadRequest, NoArgsRequest

__all__ = ["GetSnapshot", "RequestSnapshot", "DownloadSnapshot"]


class GetSnapshot(NoArgsRequest):

    def __init__(self, snapshot_id: str):
        super().__init__()
        self.snapshot_id = snapshot_id
        self.api = "/v2/sp/snapshots/%s" % self.snapshot_id
        self.method = "GET"


class DownloadSnapshot(DownloadRequest, NoArgsRequest):
    def __init__(self, link: str, dest_file: str = None):
        super().__init__()
        self.api = link
        self.method = "GET"
        self.stream = True
        self.dest_file = dest_file


class RequestSnapshot(AbstractRequest):

    default_state_filters = ["enabled", "paused"]

    def __init__(self, snapshot_type: str, state_filters: list = None):
        super().__init__()
        self.snapshot_type = snapshot_type
        self.state_filters = state_filters and state_filters or self.default_state_filters
        self.api = "/v2/sp/%s/snapshot" % self.snapshot_type
        self.method = "POST"

    def build_args(self):
        return {
            "stateFilter": ",".join(self.state_filters)
        }



import requests
from .abstract_request import AbstractRequest
from .config import get_endpoint


class Client(object):

    def __init__(self, client_id,  access_token, profile_id=None, marketplace="US", timeout=6, raise_for_status=True):
        self.client_id = client_id
        self.access_token = access_token
        self.profile_id = profile_id
        self.timeout = 6
        self.raise_for_status = raise_for_status

        # get api endpoint
        self.api_url = get_endpoint(marketplace, "API")
        self.marketplace = marketplace

    def build_url(self, endpoint: str):
        if endpoint.startswith("http"):
            return endpoint
        return self.api_url + endpoint

    def make_default_headers(self):
        headers = {
            "Content-Type": "application/json",
            "Authorization": "Bearer %s" % self.access_token,
            "Amazon-Advertising-API-ClientId": self.client_id,
        }
        if self.profile_id is not None:
            headers.update(
                {
                    "Amazon-Advertising-API-Scope": self.profile_id
                }
            )
        return headers

    def build_request(self, url, request: AbstractRequest) -> requests.Request:
        # build request args
        body = request.build_args()

        method = request.method.upper()
        req = requests.Request(method, url)
        if body:
            if req.method in ["POST", "PUT", "PATH"]:
                req.json = body
            else:
                req.params = body
        req.headers = self.make_default_headers()

        return req

    def send(self, request: AbstractRequest) -> requests.Response:
        # build url
        url = self.build_url(request.api)

        # build requests.Request and send it
        req = self.build_request(url, request)

        prepped = req.prepare()
        s = requests.Session()
        resp = s.send(prepped, stream=request.stream, timeout=self.timeout)

        if self.raise_for_status == True:
            resp.raise_for_status()

        try:
            build_response = getattr(request, "build_response")
            if callable(build_response):
                return build_response(resp)
        except AttributeError:
            pass

        return resp



from advertising import AbstractRequest, NoArgsRequest


class GetProductAdRequest(NoArgsRequest):
    def __init__(self, ad_id: str):
        super().__init__()
        self.ad_id = ad_id
        self.api = "/v2/sp/productAds/%s" % self.ad_id
        self.method = "GET"


class GetProductAdExtendedRequest(NoArgsRequest):
    def __init__(self, ad_id: str):
        super().__init__()
        self.ad_id = ad_id
        self.api = "/v2/sp/productAds/extended/%s" % self.ad_id
        self.method = "GET"


class CreateProductAdsRequest(AbstractRequest):
    def __init__(self, product_ads: list = None):
        super().__init__()
        self.product_ads = product_ads and product_ads or []
        self.apt = "/v2/sp/productAds"
        self.method = "POST"

    def add_product_ad(self, product_ad:dict):
        self.product_ads.append(product_ad)

    def build_args(self):
        return self.product_ads


class UpdateProductAdsRequest(CreateProductAdsRequest):
    def __init__(self, product_ads: list = None):
        super().__init__(product_ads)
        self.method = "PUT"


class ArchiveProductAdRequest(NoArgsRequest):
    def __init__(self, product_ad: str):
        super().__init__()
        self.product_ad = product_ad
        self.api = "/v2/sp/productAds/%s" % self.product_ad
        self.method = "DELETE"


class ListProductAdsRequest(AbstractRequest):
    def __init__(self, ad_ids: list = None, campaign_ids: list = None, ad_group_ids: list = None, states: list = None, start_index: int = 0, count: int = 100):
        super().__init__()
        self.ad_ids = ad_ids and ad_ids or []
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.ad_group_ids = ad_group_ids and ad_group_ids or []
        self.states = states and states or []
        self.start_index = start_index
        self.count = count
        self.api = "/v2/sp/productAds/"
        self.method = "GET"

    def build_args(self):

        return {
            "startIndex": self.start_index,
            "count": self.count,
            "stateFilter": ",".join(self.states),
            "campaignIdFilter":  ",".join(self.campaign_ids),
            "adGroupIdFilter":  ",".join(self.ad_group_ids),
            "adIdFilter": ",".join(self.ad_ids)
        }


class ListProductAdsExtendedRequest(ListProductAdsRequest):
    def __init__(self, ad_ids: list = None, campaign_ids: list = None, ad_group_ids: list = None, states: list = None, start_index: int = 0, count: int = 100):
        super().__init__(ad_ids=ad_ids, campaign_ids=campaign_ids,
                         ad_group_ids=ad_group_ids, states=states, start_index=start_index, count=count)
        self.api = "/v2/sp/productAds/extended"



ACCOUNT_TYPES = ["agency", "vendor", "seller"]

REGIONS = {
    "NA": {
        "AUTHORIZATION": "https://www.amazon.com/ap/oa",
        "TOKEN": "https://api.amazon.com/auth/o2/token",
        "API": "https://advertising-api.amazon.com",
    },
    "EU": {
        "AUTHORIZATION": "https://eu.account.amazon.com/ap/oa",
        "TOKEN": "https://api.amazon.co.uk/auth/o2/token",
        "API": "https://advertising-api-eu.amazon.com",
    },
    "FE": {
        "AUTHORIZATION": "https: // apac.account.amazon.com/ap/oa",
        "TOKEN": "https://api.amazon.co.jp/auth/o2/token",
        "API": "https://advertising-api-fe.amazon.com",
    },
    "SANDBOX": {
        "AUTHORIZATION": "https://www.amazon.com/ap/oa",
        "TOKEN": "https://api.amazon.com/auth/o2/token",
        "API": "https://advertising-api-test.amazon.com",
    }
}

MARKETPLACES = {
    "US": REGIONS.get("NA"),
    "CA": REGIONS.get("NA"),
    "UK": REGIONS.get("EU"),
    "FR": REGIONS.get("EU"),
    "IT": REGIONS.get("EU"),
    "DE": REGIONS.get("EU"),
    "AE": REGIONS.get("EU"),
    "JP": REGIONS.get("FE"),
    "AU": REGIONS.get("FE"),
    "SANDBOX": REGIONS.get("SANDBOX")
}


def get_endpoint(marketplace, endpoint_type=None):
    endpoints = MARKETPLACES.get(marketplace)
    if endpoints is None:
        raise ValueError("marketplace %s is not supported" % marketplace)
    if not endpoint_type:
        return endpoints

    return endpoints.get(endpoint_type)



from advertising import AbstractRequest, NoArgsRequest


class GetAdGroupBidRecommendationsRequest(NoArgsRequest):
    def __init__(self, ad_group_id: str):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.method = "GET"
        self.api = "/v2/sp/adGroups/%s/bidRecommendations" % self.ad_group_id


class GetKeywordBidRecommendationsRequest(NoArgsRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.method = "GET"
        self.api = "/v2/sp/keywords/%s/bidRecommendations" % self.keyword_id


class CreateKeywordBidRecommendationsRequest(AbstractRequest):
    """
    createKeywordBidRecommendations
    POST /v2/sp/ksponsoredeywords/bidRecommendations
    body.txt
    {
    "adGroupId": 264272438240075,
    "keywords": [
        {
        "keyword": "keyword one",
        "matchType": "broad"
        },
        {
        "keyword": "keyword two",
        "matchType": "broad"
        }
    ]
    }
    """

    def __init__(self, keyword_text: str):

        super().__init__()
        self.keyword_text = keyword_text
        self.api = "/v2/sp/sponsoredKeywords/bidRecommendations"
        self.method = "POST"

    def build_args(self):
        return None


class GetBidRecommendationsRequest(AbstractRequest):
    """
    Targeting Bid Recommendations
    getBidRecommendations
    POST  /v2/sp/targets/bidRecommendations
    """

    def __init__(self, ad_group_id: str, expressions: list):
        super().__init__()
        self.ad_group_id = ad_group_id
        self.expressions = expressions
        self.api = "/v2/sp/targets/bidRecommendations"
        self.method = "POST"

    def build_args(self):
        return {
            "adGroupId": self.ad_group_id,
            "expressions": self.expressions
        }



from advertising import AbstractRequest, NoArgsRequest

__all__ = ["ListProfilesRequest", "GetProfileRequest", "UpdateProfileRequest"]


class ListProfilesRequest(NoArgsRequest):
    def __init__(self):
        super().__init__()
        self.api = "/v2/profiles"
        self.method = "GET"


class GetProfileRequest(NoArgsRequest):
    def __init__(self, profile_id: str):
        super().__init__()
        self.profile_id = profile_id
        self.api = "/v2/profiles/%s" % self.profile_id
        self.method = "GET"


class UpdateProfileRequest(AbstractRequest):
    def __init__(self, profiles: list = None):
        super().__init__()
        self.api = "/v2/profiles"
        self.method = "PUT"
        self.profiles = profiles and profiles or []

    def add_profile(self, profile: dict):
        self.profiles.append(profile)

    def build_args(self):
        return self.profiles



from advertising import AbstractRequest, NoArgsRequest

__all__ = ["GetTargetingRequest", "GetTargetingExtendedRequest", "ListTargetingRequest",
           "ListTargetingExtendedRequest", "ListTargetingExtendedRequest", "CreateTargetingRequest",
           "CreateTargetingRecommendations", "UpdateTargetingRequest", "ArchiveTargetingRequest", "GetTargetingCategoriesRequest",
           "GetTargetingCategoriesRefinementsRequest", "GetBrandRecommendationsRequest", "GetNegativeTargetingRequest", "GetNegativeTargetingExtendedRequest",
           "ListNegativeTargetingRequest", "ListNegativeTargetingExtendedRequest", "CreateNegativeTargetingRequest", "ArchiveNegativeTargetingRequest", "UpdateNegativeTargetingRequest"
           ]


class GetTargetingRequest(NoArgsRequest):
    def __init__(self, target_id: str):
        super().__init__()
        self.target_id = target_id
        self.api = "/v2/sp/targets/%s" % self.target_id
        self.method = "GET"


class GetTargetingExtendedRequest(GetTargetingRequest):
    def __init__(self, target_id: str):
        super().__init__(target_id)
        self.api = "/v2/sp/targets/extended/%s" % self.target_id


class ListTargetingRequest(AbstractRequest):
    def __init__(self, target_ids: list = None, campaign_ids: list = None, ad_group_ids: list = None, states: list = None, category_id: str = None, start_index: int = 0, count: int = 100):
        super().__init__()
        self.target_ids = target_ids and target_ids or []
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.ad_group_ids = ad_group_ids and ad_group_ids or []
        self.states = states and states or []
        self.start_index = start_index
        self.category_id = category_id
        self.count = count
        self.api = "/v2/sp/targets"
        self.method = "GET"

    def build_args(self):

        return {
            "startIndex": self.start_index,
            "count": self.count,
            "stateFilter": ",".join(self.states),
            "campaignIdFilter":  ",".join(self.campaign_ids),
            "adGroupIdFilter":  ",".join(self.ad_group_ids),
            "targetIdFilter": ",".join(self.target_ids),
            "categoryId": self.category_id
        }


class ListTargetingExtendedRequest(ListTargetingRequest):

    def __init__(self, target_ids: list = None, campaign_ids: list = None, ad_group_ids: list = None, states: list = None, start_index: int = 0, count: int = 100):
        super().__init__(target_ids=target_ids, campaign_ids=campaign_ids,
                         ad_group_ids=ad_group_ids, states=states, start_index=start_index, count=count)
        self.api = "/v2/sp/targets/extended"


class CreateTargetingRequest(AbstractRequest):
    def __init__(self, targetings: list = None):
        super.__init__()
        self.targetings = targetings and targetings or []
        self.api = "/v2/sp/targets/"
        self.method = "POST"

    def add_targeting(self, targeting:dict):
        self.targetings.append(targeting)

    def build_args(self):
        return self.targetings


class CreateTargetingRecommendations(AbstractRequest):
    def __init__(self, recommendations: list = None):
        super.__init__()
        self.recommendations = recommendations and recommendations or []
        self.api = "/v2/sp/targets/productRecommendations"
        self.method = "POST"

    def add_recommendation(self, recommendation:dict):
        self.recommendations.append(recommendation)

    def build_args(self):
        return self.recommendations


class UpdateTargetingRequest(CreateTargetingRequest):
    def __init__(self, targetings: list = None):
        super.__init__(targetings)
        self.method = "PUT"


class ArchiveTargetingRequest(CreateTargetingRequest):
    def __init__(self, targetings: list = None):
        super.__init__(targetings)
        self.method = "DELETE"


class GetTargetingCategoriesRequest(AbstractRequest):
    def __init__(self, asins: list = None):
        super.__init__()
        self.api = "/v2/sp/targets/categories"
        self.method = "GET"
        self.asins = asins and asins or []

    def build_args(self):
        return {
            "asins": ",".join(self.asins)
        }


class GetTargetingCategoriesRefinementsRequest(AbstractRequest):
    def __init__(self, category_id: int = None):
        super.__init__()
        self.api = "/v2/sp/targets/categories/refinements"
        self.method = "GET"
        self.category_id = category_id

    def build_args(self):
        return {
            "categoryId":  self.category_id
        }


class GetBrandRecommendationsRequest(AbstractRequest):
    def __init__(self, keyword: str = None, category_id: int = None):
        super.__init__()
        self.api = "/v2/sp/targets/brands"
        self.method = "GET"
        self.keyword = keyword
        self.category_id = category_id

    def build_args(self):
        return {
            "keyword": self.keyword,
            "categoryId":  self.category_id
        }


class GetNegativeTargetingRequest(NoArgsRequest):
    def __init__(self, target_id: str = None):
        super.__init__()
        self.target_id = target_id
        self.api = "/v2/sp/negativeTargets/%s" % self.target_id
        self.method = "GET"


class GetNegativeTargetingExtendedRequest(NoArgsRequest):
    def __init__(self, target_id: str = None):
        super.__init__()
        self.target_id = target_id
        self.api = "/v2/sp/negativeTargets/extended/%s" % self.target_id
        self.method = "GET"


class ListNegativeTargetingRequest(ListTargetingRequest):

    def __init__(self, target_ids: list = None, campaign_ids: list = None, ad_group_ids: list = None, states: list = None, start_index: int = 0, count: int = 100):
        super().__init__(target_ids=target_ids, campaign_ids=campaign_ids,
                         ad_group_ids=ad_group_ids, states=states, start_index=start_index, count=count)
        self.api = "/v2/sp/negativeTargets"


class ListNegativeTargetingExtendedRequest(ListTargetingRequest):

    def __init__(self, target_ids: list = None, campaign_ids: list = None, ad_group_ids: list = None, states: list = None, start_index: int = 0, count: int = 100):
        super().__init__(target_ids=target_ids, campaign_ids=campaign_ids,
                         ad_group_ids=ad_group_ids, states=states, start_index=start_index, count=count)
        self.api = "/v2/sp/targets/extended"


class CreateNegativeTargetingRequest(AbstractRequest):
    def __init__(self, negative_targetings: list = None):
        super.__init__()
        self.negative_targetings = negative_targetings and negative_targetings or []
        self.method = "POST"
        self.api = "/v2/sp/negativeTargets"

    def add_negative_targeting(self, negative_targeting:dict):
        self.negative_targetings.append(negative_targeting)

    def build_args(self):
        return self.negative_targetings


class ArchiveNegativeTargetingRequest(CreateNegativeTargetingRequest):
    def __init__(self, targetings: list = None):
        super.__init__(targetings)
        self.method = "DELETE"


class UpdateNegativeTargetingRequest(CreateNegativeTargetingRequest):
    def __init__(self, targetings: list = None):
        super.__init__(targetings)
        self.method = "PUT"



from advertising import AbstractRequest, NoArgsRequest

__all__ = []


class ListPortfoliosRequest(AbstractRequest):
    def __init__(self, portfolio_id: list = None, portfolio_name: str = None, portfolio_state: str = None):
        super().__init__()
        self.api = "/v2/portfolios"
        self.method = "GET"
        self.portfolio_id = portfolio_id
        self.portfolio_name = portfolio_name
        self.portfolio_state = portfolio_state

    def build_args(self):
        return {
            "portfolioStateFilter": self.portfolio_state,
            "portfolioNameFilter": self.portfolio_name,
            "portfolioIdFilter": self.portfolio_id,
        }


class ListPortfoliosExtendedRequest(ListPortfoliosRequest):
    def __init__(self, portfolio_id: list = None, portfolio_name: str = None, portfolio_state: str = None):
        super().__init__(portfolio_id, portfolio_name, portfolio_state)
        self.api = "/v2/portfolios/extended"


class GetPortfolioRequest(NoArgsRequest):
    def __init__(self, protfolio_id: str):
        super.__init__()
        self.protfolio_id = protfolio_id
        self.api = "/v2/portfolios/%s" % self.protfolio_id
        self.method = "GET"


class GetPortfolioExtendedRequest(NoArgsRequest):
    def __init__(self, protfolio_id: str):
        super.__init__()
        self.protfolio_id = protfolio_id
        self.api = "/v2/portfolios/extended/%s" % self.protfolio_id
        self.method = "GET"


class CreateProtfoliosRequest(AbstractRequest):
    def __init__(self, portfolios: list = None):
        super.__init__()
        self.portfolios = portfolios
        self.api = "/v2/portfolios"
        self.method = "POST"

    def add_protfolio(self, protfolio: dict):
        self.portfolios.append(protfolio)

    def build_args(self):
        return self.portfolios


class UpdateProtfoliosRequest(CreateProtfoliosRequest):
    def __init__(self, portfolios: list = None):
        super.__init__(portfolios)
        self.method = "PUT"



from advertising import AbstractRequest, NoArgsRequest


class GetCampaignNegativeKeywordRequest(NoArgsRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.method = "GET"
        self.api = "/v2/sp/campaignNegativeKeywords/%s" % self.keyword_id


class GetCampaignNegativeKeywordExtendedRequest(NoArgsRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.method = "GET"
        self.api = "/v2/sp/campaignNegativeKeywords/extended/%s" % self.keyword_id


class CreateCampaignNegativeKeywordsRequest(AbstractRequest):
    def __init__(self, keywords: list = None):
        super().__init__()
        self.keywords = keywords and keywords or []
        self.api = "/v2/sp/campaignNegativeKeywords"
        self.method = "POST"

    def add_keyword(self, keyword: dict):
        self.keywords.append(keyword)

    def build_args(self):
        return self.keywords


class UpdateCampaignNegativeKeywordsRequest(CreateCampaignNegativeKeywordsRequest):

    def __init__(self, keywords: list = None):
        super().__init__()
        self.keywords = keywords and keywords or []
        self.api = "/v2/sp/campaignNegativeKeywords"
        self.method = "PUT"


class ArchiveCampaignNegativeKeywordRequest(AbstractRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.api = "/v2/sp/campaignNegativeKeywords/%s" % self.keyword_id
        self.method = "DELETE"


class ListCampaignNegativeKeywordsRequest(AbstractRequest):

    def __init__(self,
                 keyword_text: str = None,
                 match_type: list = None,
                 campaign_ids: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__()
        self.keyword_text = keyword_text
        self.match_type = match_type and match_type or []
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.start_index = start_index
        self.count = count
        self.api = "/v2/sp/campaignNegativeKeywords/"
        self.method = "GET"

    def build_args(self):

        return {
            "startIndex": self.start_index,
            "count": self.count,
            "keywordText": self.keyword_text,
            "matchTypeFilter": ",".join(self.match_type),
            "campaignIdFilter":  ",".join(self.campaign_ids),
        }


class ListCampaignNegativeKeywordsExtendedRequest(ListCampaignNegativeKeywordsRequest):

    def __init__(self,
                 keyword_text: str = None,
                 match_type: list = None,
                 campaign_ids: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):

        super().__init__(keyword_text=keyword_text, match_type=match_type,
                         campaign_ids=campaign_ids,
                         start_index=start_index, count=count)
        self.api = "/v2/sp/campaignNegativeKeywords/extended"
        self.method = "GET"



from setuptools import setup, find_packages
from codecs import open
from os import path

current_path = path.abspath(path.dirname(__file__))

with open(path.join(current_path, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='amazon_advertising',
    version='1.0.3',
    description='simple Amazon Advertising API client',
    long_description=long_description,
    url='https://gitlab.com/ketu.lai/amazon-advertising',
    author='ketu.lai',
    author_email='ketu.lai@gmail.com',
    license='MIT',
    keywords='amazon advertising api',
    packages=find_packages(),
    install_requires=[],
    tests_require=[],
    python_requires='>=3.6',
)



from advertising import AbstractRequest

__all__ = ["GetKeywordRecommendations"]


class GetKeywordRecommendations(AbstractRequest):

    def __init__(self, asins: list = None, url: str = None, max_num_suggestion: int = 100):
        super().__init__()
        self.asins = asins
        self.url = url
        self.max_num_suggestion = max_num_suggestion
        self.api = "/sb/recommendations/keyword"
        self.method = "POST"

    def build_args(self):
        if self.asins is None and self.url is None:
            raise ValueError("Specifies the url or asins")

        args = {
            "maxNumSuggestions": self.max_num_suggestion,
            "asins": self.asins,
            "url": self.url
        }
        return args



from advertising import AbstractRequest, NoArgsRequest


class GetBiddableKeywordRequest(NoArgsRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.method = "GET"
        self.api = "/v2/sp/keywords/%s" % self.keyword_id


class GetBiddableKeywordExtendedRequest(NoArgsRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.method = "GET"
        self.api = "/v2/sp/keywords/extended/%s" % self.keyword_id


class CreateKeywordsRequest(AbstractRequest):
    def __init__(self, keywrods: list = None):
        super().__init__()
        self.keywords = keywrods and keywrods or []
        self.method = "POST"
        self.api = "/v2/sp/keywords"

    def add_keywrod(self, keyword: dict):
        self.keywords.append(keyword)

    def build_args(self):
        return self.keywords


class UpdateKeywordsRequest(CreateKeywordsRequest):
    def __init__(self, keywrods: list = None):
        super().__init__(keywrods=keywrods)
        self.method = "PUT"


class ArchiveKeywordRequest(NoArgsRequest):
    def __init__(self, keyword_id: str):
        super().__init__()
        self.keyword_id = keyword_id
        self.method = "DELETE"
        self.api = "/v2/sp/keywords/%s" % self.keyword_id


class ListBiddableKeywordsRequest(AbstractRequest):
    def __init__(self,
                 keyword_text: str = None,
                 keyword_ids: list = None,
                 match_type: list = None,
                 states: list = None,
                 campaign_ids: list = None,
                 ad_group_ids: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__()
        self.keyword_text = keyword_text
        self.match_type = match_type and match_type or []
        self.keyword_ids = keyword_ids and keyword_ids or []
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.ad_group_ids = ad_group_ids and ad_group_ids or []
        self.states = states and states or []
        self.start_index = start_index
        self.count = count
        self.api = "/v2/sp/keywords/"
        self.method = "GET"

    def build_args(self):

        return {
            "startIndex": self.start_index,
            "count": self.count,
            "keywordText": self.keyword_text,
            "matchTypeFilter": ",".join(self.match_type),
            "stateFilter": ",".join(self.states),
            "keywordIdFilter":  ",".join(self.keyword_ids),
            "campaignIdFilter":  ",".join(self.campaign_ids),
            "adGroupIdFilter":  ",".join(self.ad_group_ids),
        }


class ListBiddableKeywordsExtendedRequest(ListBiddableKeywordsRequest):
    def __init__(self,
                 keyword_text: str = None,
                 keyword_ids: list = None,
                 match_type: list = None,
                 states: list = None,
                 campaign_ids: list = None,
                 ad_group_ids: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__(keyword_text=keyword_text,
                         keyword_ids=keyword_ids,
                         match_type=match_type,
                         states=states,
                         campaign_ids=campaign_ids,
                         ad_group_ids=ad_group_ids,
                         start_index=start_index,
                         count=count)



from advertising import AbstractRequest, NoArgsRequest


class GetCampaignRequest(NoArgsRequest):
    def __init__(self, campaign_id: str):
        super().__init__()
        self.campaign_id = campaign_id
        self.api = "/v2/sp/campaigns/%s" % self.campaign_id
        self.method = "GET"


class GetCampaignExtendedRequest(NoArgsRequest):
    def __init__(self, campaign_id: str):
        super().__init__()
        self.campaign_id = campaign_id
        self.api = "/v2/sp/campaigns/extended/%s" % self.campaign_id
        self.method = "GET"


class CreateCampaignsRequest(AbstractRequest):
    def __init__(self, portfolio_id: str, campaigns: list = None):
        super().__init__()
        self.portfolio_id = portfolio_id
        self.campaigns = campaigns and campaigns or []
        self.api = "/v2/sp/campaigns"
        self.method = "POST"

    def add_campaign(self, campaign: dict):
        self.campaigns.append(campaign)

    def build_args(self):
        return {"portfolioId": self.portfolio_id,  "campaigns": self.campaigns}


class UpdateCampaignsRequest(AbstractRequest):
    def __init__(self, campaigns: list = None):
        super().__init__()
        self.campaigns = campaigns
        self.method = "PUT"

    def add_campaign(self, campaign: dict):
        self.campaigns.append(campaign)

    def build_args(self):
        return self.campaigns


class ArchiveCampaignRequest(NoArgsRequest):
    def __init__(self, campaign_id: str):
        super().__init__()
        self.campaign_id = campaign_id
        self.api = "/v2/sp/campaigns/%s" % self.campaign_id
        self.method = "DELETE"


class ListCampaignsRequest(AbstractRequest):
    def __init__(self,
                 name: str = None,
                 portfolio_id: str = None,
                 campaign_ids: list = None,
                 states: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__()
        self.name = name
        self.portfolio_id = portfolio_id
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.states = states and states or []
        self.start_index = start_index
        self.count = count
        self.api = "/v2/sp/campaigns"
        self.method = "GET"

    def build_args(self):

        return {
            "startIndex": self.start_index,
            "count": self.count,
            "name": self.name,
            "portfolioIdFilter": self.portfolio_id,
            "stateFilter": ",".join(self.states),
            "campaignIdFilter":  ",".join(self.campaign_ids)
        }


class ListCampaignsExtendedRequest(ListCampaignsRequest):
    def __init__(self,
                 name: str = None,
                 portfolio_id: str = None,
                 campaign_ids: list = None,
                 states: list = None,
                 start_index: int = 0,
                 count: int = 100
                 ):
        super().__init__()
        self.name = name
        self.portfolio_id = portfolio_id
        self.campaign_ids = campaign_ids and campaign_ids or []
        self.states = states and states or []
        self.start_index = start_index
        self.count = count
        self.api = "/v2/sp/campaigns/extended"
        self.method = "GET"
